#if defined( Uses_IconCommands ) && !defined( __IconCommands )
#define __IconCommands

const unsigned cmMinimize = 0xFFF0;

#endif

// ------------------------------------------------------------------
//	Class	TIcon
//	Base	TView
//
// TIcon is the view that is displayed when the TMinWindow is minimized.
//	The default icon will just display a small white rectangle on the
//	screen, but you can change its appearance with the draw() method.
//	A TIcon view performs the following actions:
//		* If it is currently selected, you can restore the window by
//        pressing ENTER or double-clicking with the mouse.
//		* You can move the icon like a normal TWindow (except that
//		  the "hotspot" for moving with the mouse is any area within
//		  icon, not just the top line.
// ------------------------------------------------------------------

#if defined( Uses_TIcon ) && !defined( __TIcon )
#define __TIcon

class far TRect;
class far TEvent;
class far TPalette;
class far TMinWindow;
class far opstream;
class far ipstream;
class far TStreamable;

class TIcon : public TView
	{
	public:

		friend TMinWindow;

		TIcon( const TRect& bounds );

		virtual void shutDown();
		virtual void handleEvent( TEvent& event );
		virtual void setState( ushort aState, Boolean enable );
		virtual TPalette& getPalette() const;
		virtual void draw();
		virtual Boolean valid( ushort command );

		virtual void close();
		virtual void zoom();

	protected:

		TMinWindow *link;
		Boolean linkIsValid();

	private:

		virtual const char *streamableName() const
			{ return name; }

	protected:

		TIcon( StreamableInit );
		virtual void write( opstream& os );
		virtual void *read( ipstream& is );

	public:

		static const char * const near name;
		static TStreamable *build();

	};

inline ipstream& operator >> ( ipstream& is, TIcon& cl )
	{ return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TIcon*& cl )
	{ return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TIcon& cl )
	{ return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TIcon* cl )
	{ return os << (TStreamable *)cl; }

#endif	// __TIcon

// ------------------------------------------------------------------
//	Class 	TMinFrame
//	Base    TFrame
//
//	TMinFrame is a TFrame-derived class that allows you to "minimize"
//	(or display as an icon) TWindow objects. It replaces the normal
//	TFrame that the TWindow would use. If you compare this class
//	with TFrame, you'll see a lot of "repeated" members:
//		minInitFrame 	<=> 	TFrame::initFrame
//		minFrameChars 	<=> 	TFrame::frameChars
//		maxZoomIcon 	<=> 	TFrame::zoomIcon
//		normZoomIcon 	<=> 	TFrame::unZoomIcon
//  	minCloseIcon 	<=> 	TFrame::closeIcon
//		minDragIcon 	<=> 	TFrame::dragIcon
//		minFrameLine(...) <=> 	TFrame::frameLine(...)
//	This was necessary because all of these members were declared
//	"private:" in TFrame and cannot be accessed in a derived class
//	without changing the original TV header files.
//	Operationally, TMinFrame is similar to TFrame, but has added
//	a minimize icon (down arrow) in the zoom box which, when
//	clicked, generates a { evCommand->cmMinimize } command to the
//	owning window.
// ------------------------------------------------------------------

#if defined( Uses_TMinFrame ) && !defined( __TMinFrame )
#define __TMinFrame

class far TRect;
class far TEvent;
class far TPoint;

class TMinFrame : public TFrame
	{

	public:

		TMinFrame( const TRect& bounds );

		virtual void handleEvent( TEvent& event );
		virtual void draw();

	private:

		virtual const char *streamableName() const
			{ return name; }

	protected:

		TMinFrame( StreamableInit );

		// The following are basically extracted from the TFrame class,
		//	since in that class these members/methods are private and
		//	can't be accessed. Me and Mark Wilden (TeamB) have on-going
		//	discussions <g> about which is better: change the TV library
		//	to set these members/methods protected, or (as I have done),
		//	make accessible copies here without having to change the
		//	default library. As this copy stands, you can use this class
		//	WITHOUT having to modify TV. If, in the future, BI changes
		//	the access level in TFrame, these can go away. Until then,
		//	I'd recommend leaving them in.	-Pat-

		void minFrameLine( TDrawBuffer& frameBuf, short y, short n, uchar color );
		static const char minInitFrame[19];
		static char minFrameChars[33];
		static const char *maxZoomIcon;
		static const char *normZoomIcon;
		static const char *minCloseIcon;
		static const char *minDragIcon;

	public:

		static const char * const near name;
		static TStreamable *build();

	};

inline ipstream& operator >> ( ipstream& is, TMinFrame& cl )
	{ return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TMinFrame*& cl )
	{ return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TMinFrame& cl )
	{ return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TMinFrame* cl )
	{ return os << (TStreamable *)cl; }

#endif  // Uses_TMinFrame

// ------------------------------------------------------------------
//	Class	TMinWindow
//	Base    TWindow
//
//	This class is the base for a minimize-capable window. The capabilities
//	are provided by a TIcon (passed in the constructor), this class,
//	and a TMinFrame (returned by initFrame).
// ------------------------------------------------------------------

#if defined( Uses_TMinWindow ) && !defined( __TMinWindow )
#define __TMinWindow

class far TRect;
class far TPoint;
class far TEvent;
class far TIcon;
class far TStreamable;
class far opstream;
class far ipstream;

class TMinWindow : public TWindow
	{
	public:

		friend TIcon;

		TMinWindow( const TRect& bounds, const char *aTitle, short aNumber, TIcon *aIcon );

		virtual void shutDown();
		virtual void handleEvent( TEvent& event );
		static TFrame *initFrame( TRect bounds );

		virtual void minimize();

	protected:

		TIcon *icon;
		Boolean iconIsValid();

	private:

		virtual const char *streamableName() const
			{ return name; }

	protected:

		TMinWindow( StreamableInit );
		virtual void write( opstream& os );
		virtual void *read( ipstream& is );

	public:

		static const char * const near name;
		static TStreamable *build();

	};

inline ipstream& operator >> ( ipstream& is, TMinWindow& cl )
	{ return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TMinWindow*& cl )
	{ return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TMinWindow& cl )
	{ return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TMinWindow* cl )
	{ return os << (TStreamable *)cl; }

#endif	// __TMinWindow
