#define Uses_TMinWindow
#define Uses_TIcon
#define Uses_TPoint
#define Uses_TEvent
#define Uses_TDrawBuffer
#define Uses_TKeys
#define Uses_IconCommands
#define Uses_TMinFrame
#define Uses_opstream
#define Uses_ipstream
#define Uses_TStreamable
#define Uses_TStreamableClass
#include "main.h"

#if !defined( __STRING_H )
#include <string.h>
#endif

__link( RWindow )
const char * const near TMinWindow::name = "TMinWindow";
TStreamableClass RMinWindow( TMinWindow::name, TMinWindow::build,
	__DELTA(TMinWindow) );

TMinWindow::TMinWindow( const TRect& bounds, const char *aTitle,
  short aNumber, TIcon *aIcon ) :
  TWindow( bounds, aTitle, aNumber ),
  TWindowInit( &TMinWindow::initFrame ),
  icon(aIcon)
	{
	flags |= wfZoom;

	if( icon != 0 )
		{
		icon->link = this;
		icon->moveTo( bounds.a.x, bounds.a.y );
        }
	}

// ------------------------------------------------------------------
// When an window is told to shut down, it can't forget its linked
//	icon. The default process, if an icon is linked, is to:
//		1) NULL the icon's link member, so we don't get into a
//			vicious cycle.
//		2) Remove the icon from its TGroup owner, if it has one.
//		3) destroy the icon
//		4) destroy the window
// ------------------------------------------------------------------

void TMinWindow::shutDown()
	{
	if( iconIsValid() )
		{
		icon->link = 0;
		if( icon->owner == 0 )
			destroy(icon);
		}
	icon = 0;
	TWindow::shutDown();
	}

void TMinWindow::handleEvent( TEvent& event )
	{
	TWindow::handleEvent(event);

	// If the event evCommand->cmMinimize is received, call the
	//	virtual method minimize() and then signal the owner
	//	that this window has just been minimized.

	if( event.what == evCommand && event.message.command == cmMinimize &&
	  iconIsValid() )
		{
		TGroup *o = owner;
		minimize();
		event.what = evBroadcast;
		event.message.infoPtr = icon;
		o->handleEvent(event);
		clearEvent(event);
		}
	}

TFrame *TMinWindow::initFrame( TRect r )
	{
	return new TMinFrame(r);
	}

void TMinWindow::minimize()
	{

	// To minimize a window, we hide it and insert the icon.

	if( iconIsValid() )
		{
		TGroup *o = owner;
		o->lock();
		o->insertBefore( icon, this );
		o->remove( this );
		o->unlock();
		}
	}

Boolean TMinWindow::iconIsValid()
	{
	return Boolean( icon != 0 && icon->link == this );
	}

void TMinWindow::write( opstream& os )
	{
	TWindow::write(os);
	os << icon;
	}

void *TMinWindow::read( ipstream& is )
	{
	TWindow::read(is);
	is >> icon;
	return this;
	}

TMinWindow::TMinWindow( StreamableInit ) :
	TWindow( streamableInit ),
	TWindowInit(0)
	{
	}

TStreamable *TMinWindow::build()
	{
	return new TMinWindow( streamableInit );
	}
