#define Uses_TTitleDeskTop
#define Uses_TNSSortedCollection
#define Uses_TWindow
#define Uses_TEvent
#define Uses_IconCommands
#include "main.h"

#if !defined( __STRING_H )
#include <string.h>
#endif

// ------------------------------------------------------------------
// Need a TNSSortedCollection for use in arrangeIcons. It will store
//	TWindow pointers sorted by title.
// ------------------------------------------------------------------

class TNSSortedWindow : public TNSSortedCollection
	{
	public:

		TNSSortedWindow( ccIndex aLimit, ccIndex aDelta );

		virtual int compare( void *key1, void *key2 );
		virtual void *keyOf( void *item );
		virtual void freeItem( void * );
	};

TNSSortedWindow::TNSSortedWindow( ccIndex aLimit, ccIndex aDelta ) :
  TNSSortedCollection(aLimit,aDelta)
	{
	duplicates = True;
	shouldDelete = True;
	}

int TNSSortedWindow::compare( void *key1, void *key2 )
	{
	return strcmp( (char *)key1, (char *)key2 );
	}

void *TNSSortedWindow::keyOf( void *item )
	{
	return (void *) ((TWindow *)item)->title;
	}

void TNSSortedWindow::freeItem( void * )
	{

	// Leave the TWindow pointer alone!

	}

// ------------------------------------------------------------------


// This function is called for all the views in the desktop
//	to ensure that p is a subview of the desktop.

static Boolean isThisView( TView *pv, void *p )
	{
	return Boolean( pv != 0 && pv == p );
	}

TTitleDeskTop::TTitleDeskTop( const TRect& bounds ) :
  TDeskTop(bounds),
  TDeskInit( &TTitleDeskTop::initBackground )
	{
	}

void TTitleDeskTop::handleEvent( TEvent& event )
	{

	// If an "owned" TView object has been minimized, call the
	// 	arrangeIcons() method to rearrange all of the icons in
	//  the desktop.

	if( event.what == evBroadcast &&
	  event.message.command == cmMinimize &&
	  event.message.infoPtr != 0 &&
	  firstThat( isThisView, event.message.infoPtr ) != 0 )
		{
		arrangeIcons();
		clearEvent(event);
		}

	TDeskTop::handleEvent(event);
	}

// This function is called to ensure that p is an icon.

static Boolean viewIsIcon( TView *p )
	{
	TEvent event;

	event.what = evCommand;
	event.message.command = cmIconsEat;
	event.message.infoPtr = 0;
	p->handleEvent(event);
	return Boolean( event.what != evCommand );
	}

// Arrange the icons at the bottom of the desktop in alphabetical
//	order.

void TTitleDeskTop::arrangeIcons()
	{
	TNSSortedWindow *coll = new TNSSortedWindow( 10, 2 );
	TRect r = getExtent();

	// Calculate the region needed to hold the icons.
	// Since ALL icons are TTitleFrame's with a
	// size of 14x1, use this size to calculate.
	// Allows 4 icons per row.

	TView *p = last;
	while( (p = p->next) != last )
		if( viewIsIcon(p) )
			coll->insert(p);

	short n = coll->getCount();
	short rows = n/4 + ((n % 4) ? 1 : 0);
	short x = 1;
	short y = r.b.y - 2*rows;

	lock();

	for( n = 0; n < coll->getCount(); n++ )
		{
		p = (TView *) coll->at(n);
		if( p != 0 )
			{
			p->moveTo( x,y );
			x += 17;
			if( x+16 >= r.b.x )
				{
				x = 1;
				y += 2;
				}
			}
		}

	unlock();

	destroy(coll);
	}
